function [data] = pla_path(param,im,data,ll,kk)

%extenxion of tight boundingbox
EXTENSION         =  7;
CONTRAST          =  3;
CUTOFF            =  2;
ITERATIONS        = 25;
SAMPLINGSDISTANCE = .1;
DAMPING           =  1;   % 1 means no damping

%get outer boundingbox of label
msr = measure(im.mask{ll},[],{'Minimum','Maximum'});

%im.mask{ll}

%upperleft corner and lower right corner of image
TL=[0,0];
BR=size(im.flat)-1;

%upperleft corner and lower right corner of image
tl = msr.Minimum'-[1,1]*EXTENSION;
br = msr.Maximum'+[1,1]*EXTENSION;

%keep within original image
tl(tl < TL) = TL(tl < [0,0]);
br(br > BR) = BR(br > BR);

org = tl;
sz  = br-tl;

%create crop image and correct enpoint for offset
crop.im     = dip_crop(im.flat, org, sz);
crop.offset = tl;
crop.crdA   = im.crdA{ll} - tl;
crop.crdB   = im.crdB{ll} - tl;
crop.mask   = dip_crop(im.mask{ll}, org, sz); 
crop.skel   = dip_crop(im.sk{ll}, org, sz); 

%
cost = CONTRAST-((crop.im-min(crop.im))/(max(crop.im)-min(crop.im)))*(CONTRAST-1);
costfirst = cost+dt(~crop.skel).^4;

try
    
  tmpdata = path_by_iterative_fm_graddescent(...
      cost,...
      crop.crdA,...
      crop.crdB,...
      DAMPING,...
      ITERATIONS,...
      CUTOFF,...
      SAMPLINGSDISTANCE, ...
      costfirst,...
      crop.mask);

  
  
  if isempty(data(im.condition).path{1})
    cnt = 1;
  else
    cnt = size(data(im.condition).path,2)+1;
  end

  fprintf('      %d th good path for condition %d\n',cnt, im.condition);
  data(im.condition).path{cnt}  = tmpdata;
  data(im.condition).resolution(cnt) = param.condition(im.condition).param{im.image};
  
  %create overlay
  
  %output file
  fn=[param.dir.output filesep param.condition(im.condition).name{1} '.ps'];
  
  %delete output file if it already exist 
  if(cnt == 1 && exist(fn,'file'))
    delete(fn);
    fprintf('\rerase file %s in %s\n',fn,mfilename);
  end
  
  %create overlay
  h=dipshow(crop.im,'lin');
  set(h,'visible','off')
  diptruesize(h,400);
  hold on
  hha(1)=plot(tmpdata{1}(:,1),tmpdata{1}(:,2),'b');
  hha(2)=plot(tmpdata{end}(:,1),tmpdata{end}(:,2),'r');%not if the centeroff gravity is on then this is centerof gravity otherwise the last iteration of the fm procedure
  
  text(0,0,fn,'VerticalAlignment','top','color','yellow','interpreter','none','fontsize',2);
  text(0,10,[num2str(im.condition),'/',num2str(im.image),'/',num2str(kk),'/',num2str(ll)],'VerticalAlignment','top','color','yellow','interpreter','none','fontsize',5);
  text(0,20,['data(' num2str(im.condition) ').path(' num2str(cnt) ')'],'VerticalAlignment','top','color','yellow','interpreter','none','fontsize',5);
  
  plot([1,5],[2,2],'b');
  plot([1,5],[4,4],'r');
  
  text(6,2, 'first','VerticalAlignment','middle','color','yellow','interpreter','none','fontsize',5);
  text(6,4, 'final','VerticalAlignment','middle','color','yellow','interpreter','none','fontsize',5);
  
  print(h,fn,'-dpsc2','-append');
  
  %destroy window
  delete(h);


%try
catch
  fprintf('\n');
  disp(['descent failed ignoring path in ',mfilename]);
end


function it=path_by_iterative_fm_graddescent(im,endcoord, startcoord,damping,iterations,cutoff,sampling,costfirst,immask)

% damping ::    should correction be damped to sircumvent ocillations
% iterations :: number of iterations to be performed
% cutoff ::     piece of the streamline which is replaced by end points
%               to garruntee arrival at endpoints
% coord ::      start end end points of the paths to find
% im ::         original cost image
% sampling :: samplings diatnce of ponts on path
% immask region to which path is confined

SAMPLINGDISTANCE = sampling;%.01;
CUTOFFDISTANCE   = cutoff;
D                = damping;%1
N                = iterations;%25

[stream]     = getstream(costfirst,  endcoord, startcoord,CUTOFFDISTANCE,SAMPLINGDISTANCE);
p.path       = stream;
p.endpoint   = endcoord;
p.startpoint = startcoord;

it    = deform2(im,p,D,N,CUTOFFDISTANCE,SAMPLINGDISTANCE,immask);
it{1} = stream;

%all data can be stacked together as all are under the same
%conditions if the scaling is constant

%##########################################################################################
%##########################################################################################
%##########################################################################################


function [stream] = getstream(im, startcoord, endcoord, CUTOFFDISTANCE,SAMPLINGDISTANCE);

%cost image
im    = dip_image(im,'sfloat');
mask  = ~bdilation(newim(im,'bin'),1,1,1);
mask  = dip_image(mask,'bin8');

%seed points: empty in this case as we can use suppixel start points
seed  = newim(im,'bin');

%first and second entry of cost image contains seed points
im(0)            = startcoord(1);
im(1)            = startcoord(2);
[time,dist,prop] = dip_fm(im,mask,seed,0,[],[],[]);

xc = xx(im,'corner');
yc = yy(im,'corner');

sigmagradient = 1;

xd = dx(time,sigmagradient);
yd = dy(time,sigmagradient);
gm = gradmag(time,sigmagradient);

gm(gm==0) = 1;%avoid division by zero
xd        = xd/gm;
yd        = yd/gm;

%endcoords
streamx = endcoord(1);
streamy = endcoord(2);

streams = dip_stream2(xc,yc,-xd,-yd,streamx,streamy,[SAMPLINGDISTANCE,20000]);

%cutting off the stream to get rid of  oscilations
mask   = sqrt((streams{1}(:,1)-startcoord(1)).^2+(streams{1}(:,2)-startcoord(2)).^2)>CUTOFFDISTANCE;
stream = streams{1}(find(mask),:);

%end

%##########################################################################################
%##########################################################################################
%##########################################################################################


function newpath = deform2(strand,p,D,N, CUTOFFDISTANCE,SAMPLINGDISTANCE,immask)

if 0
  D=1;
  N=25;
end

[s.x,s.y,s.l] = findspline(p.path, 0, 0, 0.1);

for n = 1:N
  s.length   = s.l(end);
  s.sampling = [0:s.length];

  [s.dxdy, s.p, subpix] = get_loodlijnen(s.x, s.y, s.sampling);

  
  if 0
    if n>1
      if n==2
	dipshow(111,strand,'lin');
      end
      figure(111)
      hold on
      plot(newpath{n}(:,1),newpath{n}(:,2))
      hold off
    end
  end
  
  %check if path stays within mask
  if n>1 && ...
	any(~double(immask(round(newpath{n}(:,1))*size(strand,2)+round(newpath{n}(:,2)))));
    error(['path runs out of mask:: ignore strand']);
  end

  sub.x = subpix.x(:);
  sub.y = subpix.y(:);
  
  %detect coordinates outside image domain -> put them to the
  %maxium of resampled image
  coordoutofbounds = ...
      sub.x<0|...
      sub.x>size(strand,1)-1|...
      sub.y<0|...
      sub.y>size(strand,2)-1;
  
  sub.x(coordoutofbounds) = 1;
  sub.y(coordoutofbounds) = 1;

  getsubpixel                   = get_subpixel(strand,[sub.x,sub.y],'cubic');
  getsubpixel(coordoutofbounds) = max(getsubpixel);
  ds.im                         = reshape(dip_image(getsubpixel),size(subpix.x));

  ds.im(isnan(ds.im)) = max(ds.im);%failsafe

  
  ds.im              = extend(ds.im,size(ds.im)+[0 20],'symmetric');
  ds.im(:,0:9)       = repmat(ds.im(:,10),[1 10]);
  ds.im(:,end-9:end) = repmat(ds.im(:,end-10),[1 10]);

  ds.stream = getstream(ds.im, [10; 10], [10; size(s.p,2)+9],CUTOFFDISTANCE,SAMPLINGDISTANCE);

  
  [ds.x,ds.y,ds.l] = findspline(ds.stream, [10;10], 0, SAMPLINGDISTANCE);

  ds.length = ds.l(end);

  [ds.p ds.p_index] =  min_dist_to_line([ppval(ds.x,-1:0.1:(ds.length+1)); ppval(ds.y,-1:0.1:(ds.length+1))], [zeros(1,size(s.p,2))+10; 10:size(s.p,2)+9], [ones(1,size(s.p,2));zeros(1,size(s.p,2))] );

  p.path          = (s.p+D*s.dxdy.*[ds.p(1,:); ds.p(1,:)])';
  p.path(end+1,:) = p.endpoint;

  
  [s.x,s.y,s.l] = findspline(p.path, 0, 0, 0.1);

  newpath{n+1} = [ppval(0:0.1:s.l(end),s.x); ppval(0:0.1:s.l(end),s.y)]';
end

%newpath = [ppval(0:0.1:s.l(end),s.x); ppval(0:0.1:s.l(end),s.y)]';



%##########################################################################################
%##########################################################################################
%##########################################################################################


function [sx,sy,l_s ] = findspline(stream, startcoord, dep, SAMPLINGDISTANCE);



%obtain some avarage path

%one way...
na=size(stream,1);
x_s=spline(1:na,stream(:,1),1:SAMPLINGDISTANCE:na);
y_s=spline(1:na,stream(:,2),1:SAMPLINGDISTANCE:na);


if (startcoord) %kan fout gaan als startcoord == [0; 4] ofzo.
		% adding end(start)coords
		x_s(end+1)=startcoord(1);
		y_s(end+1)=startcoord(2);
end;

% calculating length
dl_s=sqrt([0,(x_s(1:end-1)-x_s(2:end))].^2+[0,(y_s(1:end-1)-y_s(2:end))].^2);
l_s=cumsum(dl_s(:));



%making "it" a function of the contour lenght
%sampling=[0:floor(l_s(end))]; %ceil->floor? round?
sx = spline(l_s,x_s);
sy = spline(l_s,y_s);

%##########################################################################################
%##########################################################################################
%##########################################################################################

function [dxdy, points, subpix] = get_loodlijnen(sx, sy, sampling)

points(1,:)=ppval(sx,sampling);
points(2,:)=ppval(sy,sampling);


sdx = sx;
sdx.coefs =  sx.coefs(:,1:3)*[3 0 0; 0 2 0; 0 0 1]; 
sdx.order=3;

dxdy(2,:)= ppval(sdx,sampling);

sdy = sy;
sdy.coefs =  sy.coefs(:,1:3)*[3 0 0; 0 2 0; 0 0 1]; 
sdy.order=3;

dxdy(1,:)= -ppval(sdy,sampling);

norm =  sqrt(dxdy(1,:).^2 + dxdy(2,:).^2);
dxdy(1,:) = dxdy(1,:) ./ norm;
dxdy(2,:) = dxdy(2,:) ./ norm;

subpix.x = repmat(points(1,:),[21,1])+(dxdy(1,:)'*[-10:1:10])';
subpix.y = repmat(points(2,:),[21,1])+(dxdy(2,:)'*[-10:1:10])';




%##########################################################################################
%##########################################################################################
%##########################################################################################

function [p index] = min_dist_to_line(curve, points, dxdy)
%[p index] = min_dist_to_line(curve, points, dxdy )
%
%curve = [curve_x; curve_y]
%points = [p1_x, p2_x, ...; p1_y, p2_y, ...]
%dxdy = [dx1, dx2, ...; dy1, dy2, ...]
%
%Finds the intersections of the curve with lines defined by points and derivatives.
%When it encounters multiple intersections it returns the one closesed to the
%point. p(:,x) are the coordinates of de intersection in the coordinate system with
%the origin at points(:,x) and the x-axis defined by the line through points(:,x).
%index are de indexes of curve coresponding with the intesections.
%
%26/09/2006 -  R.J. van Heekeren

%TODO checken van arguments?

for n = 1:size(points, 2)
  %	plot(points(n)+[-5:0.01:5]*dxdy(:,n))
  if(dxdy(1,n) == 0) 
    phi = pi/2;
  elseif(dxdy(2,n) == 0)
    phi = (dxdy(1,n) > 0)*pi;
  else
    phi = pi*(dxdy(1,n) > 0) - atan(dxdy(2,n)/dxdy(1,n));
  end


  hoek(n) = phi;
  
  %TODO eventueel een van de twee gelijkstellen aan 1
  roty = [ sin(phi), cos(phi)];
  offsety = roty * points(:,n);

  rotx = [ -cos(phi), sin(phi)];
  offsetx = rotx * points(:,n);

  y_c = roty * curve - offsety;

  maxval = max(abs(y_c)) + 1; %TODO event. iest snellers bedenken
  clear y_index;

  for a = 1:20 % max 20 iteraties
    
    [val y_index(a)] = min(abs(y_c));

    % "masken" zodat hij dit minimum niet meer vindt
    y_c(max([y_index(a)-3, 1]):min([y_index(a)+3, size(y_c,2)])) = maxval;

    if(max( ( abs(y_index-y_index(a)) <=4 ) & ( abs(y_index-y_index(a)) > 0 ) ))
      y_index = y_index(1:end-1); %kan mooier?
				  %Minimum al eerder gevonden
				  break; 
    end
  end
  
  % neem het minimum met de laagste |x|-waarde
  [val a] = min(abs(rotx * curve(:,y_index) - offsetx));

  % geef index en de coordinaten in het geroteerde stelsel terug
  index(n) = y_index(a);
  p(:,n) = [rotx * curve(:,index(n)) - offsetx; roty * curve(:,index(n)) - offsety;];
end


%##########################################################################################
%##########################################################################################
%##########################################################################################

function out=dip_stream2(x,y,u,v,p,q,options)
out=stream2(double(x),double(y),double(u),double(v),double(p),double(q),options);

%##########################################################################################
%##########################################################################################
%##########################################################################################

