%##########################################################################
%#####
% create plot axes and call actual plotting function
%###############################################################################
function pla_plot(data,datahist,param,out,selection,lengths)

if( ~(isstruct(out) && ...
      isfield(out,'usesubplots') && ...
      (out.usesubplots == true || out.usesubplots == false)) )
  error(['something wrong with the variable "out" structure in ' mfilename]);      
end   


%which (sub)plots should be produced, for explanation of id's see below ? 
out.subplots.id     = [1,2,3,4,5,6,7,8,9,10,11,12,13,14];
%out.subplots.id     = [1,2,5,6,7,8,9,10,11,12];

% id :: fit 
% 1  :: <R^2> 
% 2  :: <cos>
% 3  :: Histogram
% 4  :: <angle^2>
% 5  :: <angle^2>^2/<angle^4>
% 6  :: D^4/L^4
% 7  :: Histogram lengths
% 8  :: Persistence lenght as function of L_max
% 9  :: Valle double powerlaw
%10  :: Hist 1
%11  :: Hist 2
%12  :: Hist 3
%13  :: Hist 4
%14  :: <angle^4>/<angle^2>^2
%15  :: Persistense length as function of L for short lengths,

set(0,'defaultaxesfontsize',5);
set(0,'defaulttextfontsize',5);  


if(out.usesubplots == true)
  if( (isstruct(out) && ...
       isfield(out,'subplots') && ...
       isstruct(out.subplots) && ...
       isfield(out.subplots,'subs')))
    error(['the before hand definition of "out.subplots" is not yet' ...
	   ' implemented ' mfilename]);      
  else
    %set deafault configuration for subplots
   
    %number of rows /columns used in the subplot
    out.subplots.nrows  = 5;
    out.subplots.ncols  = 4;

    %define subplot "windows"
    %out.subplots.subs{id}  = [subplot windows];
    out.subplots.subs{1}   = [1,2]; 
    out.subplots.subs{2}   = [3,4];
    out.subplots.subs{3}   = [5];
    out.subplots.subs{4}   = [10];
    out.subplots.subs{5}   = [6,7];
    out.subplots.subs{6}   = [15];
    out.subplots.subs{7}   = [11,12];
    out.subplots.subs{8}   = [13,14];
    out.subplots.subs{9}   = [16];
    out.subplots.subs{10}  = [17];
    out.subplots.subs{11}  = [18];
    out.subplots.subs{12}  = [19];
    out.subplots.subs{13}  = [8,9];
    out.subplots.subs{14}  = [20];
  end
else
    out.subplots.size{1}   = [8,4]; 
    out.subplots.size{2}   = [8,4];
    out.subplots.size{3}   = [4,4];
    out.subplots.size{4}   = [4,4];
    out.subplots.size{5}   = [8,4];
    out.subplots.size{6}   = [8,4];
    out.subplots.size{7}   = [8,4];
    out.subplots.size{8}   = [8,4];
    out.subplots.size{9}   = [4,4];
    out.subplots.size{10}  = [4,4];
    out.subplots.size{11}  = [4,4];
    out.subplots.size{12}  = [4,4];
    out.subplots.size{13}  = [8,4];
    out.subplots.size{14}  = [8,4];
end

%create figure(s) and axes
for condition = selection
  %should we produce subplots or seperate figures
  if(out.usesubplots == true)
    
    %create figure
    out.plot(condition)               = figure('visible','off');
    hold on;
    %assign subplot slots to id's
    for jj=1:length(out.subplots.id)
      out.subplots.axes(condition,jj) = subplot(out.subplots.ncols,out.subplots.nrows,out.subplots.subs{out.subplots.id(jj)});
    end
  else
    %assign figures to id's
    for jj=1:length(out.subplots.id)
      out.plot(condition,jj)          = figure('visible','off');
      out.subplots.axes(condition,jj) = gca;
    end
  end
end
set(out.plot(selection),'papertype','a4');

set(out.plot(:),'visible','off')
% call wrapper for plot routines
selection
for kk=1:length(selection)
   condition = selection(kk)
   pla__plot(data{condition},datahist{condition},param,out,condition,lengths);
end
close all
%###############################################################################
% function which call actual plot routines, i.e. wrapper
%###############################################################################
function pla__plot(data,datahist,param,out,condition,lengths)

%set sample distance for analytical plot and sample windows
param.x.sampledat = param.global.samplelengthsnm;
param.x.sampleat  = linspace(param.x.sampledat(1),param.x.sampledat(end),789);%789 just a large number to get a smooth curve


bin = min(find(floor(param.global.samplelengthsnm./(2.*datahist.hist{end}.fiterf))));

if ~isempty(bin)
  param.condition(condition).bin = bin;
else
  param.condition(condition).bin = length(param.global.samplelengthsnm);
end

% flotfunctions for each id: id=1->plotA id=2->plotB etc
%plotfunctions={@plotA,@plotB,@plotC,@plotD,@plotE,@plotF,@plotG,@plotH,@plotI,@plotJ,@plotK,@plotL,@plotM,@plotN,@plotO,@plotP,@plotQ,@plotR,@plotS,@plotT,@plotU,@plotV,@plotW,@plotX,@plotY,@plotZ};
plotfunctions={@plotA,@plotB,@plotC,@plotD,@plotE,@plotF,@plotG,@plotH,@plotI,@plotJ,@plotK,@plotL,@plotM,@plotN,@plotO,@plotP,@plotQ,@plotR,@plotS,@plotT,@plotU,@plotV,@plotW,@plotX,@plotY,@plotZ};

for ii = out.subplots.id
  plotfunctions{ii}(data,datahist,param,out,ii,condition,lengths);
end

if out.usesubplots==true
    ax = axes('position',[0,0,1,1]);
    set(ax,'color','none','Interruptible','off');
    axis off; 
    tit = param.condition(condition).name{1};
    tit(strfind(tit,'_'))=' ';
    text(0.03,.97,[param.dir.output tit],'verticalalignment','top','interpreter','none');
    set(gcf,'PaperOrientation','landscape')
    set(gcf,'Position',get(0,'Screensize'));
    set(gcf,'PaperPosition',[0,0,11.6929 8.26772]);

    fnfig = [param.dir.output filesep param.condition(condition).name{1}, '_figure_overview.fig'];
    fnps = [param.dir.output filesep param.condition(condition).name{1}, '_figure_overview.ps'];
    drawnow;
    saveas(gcf,fnfig);
    print(gcf,'-loose','-dpsc2',fnps);
    fnpsappend = [param.dir.output filesep 'overviewappend.ps'];
    print(gcf,'-append','-loose','-dpsc2',fnpsappend);
else
    for dd=1:length(out.subplots.id)
        cc=out.plot(condition,dd);
        set(cc,'papersize', out.subplots.size{dd} ,'paperunits','centimeters','paperposition',[0,0,out.subplots.size{dd}])
        ax = axes('position',[0,0,1,1]);
        set(ax,'position', [.1, .1, .8, .8],'color','none')
        axis off;
        set(cc,'visible','on')
        fnfig = [param.dir.output filesep param.condition(condition).name{1}, '_figure_' num2str(dd) '.fig'];
        fnps = [param.dir.output filesep param.condition(condition).name{1}, '_figure_' num2str(dd) '.ps'];
        drawnow;
        print(gcf,'-loose','-dpsc2',fnps);
        saveas(gcf,fnfig);
    end
end


%%
%########################################################################
%<R2>
%########################################################################
function plotA(data,datahist,param,out,id,condition,lengths)
if ~out.usesubplots
    
end
sc = 3; %i.e. 10^sc


%make axes of plot current
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on
hdat = errorbar(param.x.sampledat,...
	 data.meanrsquared.mean./(10^sc),...
	 data.meanrsquared.std./(10^sc),'r.');
hfit =plot([0,param.x.sampleat],...
     meanr2(data.plength.nm.r2.m(param.condition(condition).bin),[0,param.x.sampleat])./(10^sc),'r');
grid on;
axis tight;
axx=axis;
axis(axx.*[0,1,0,1]);

%setxtick(gca,10,param);
%th=title(['<R^2>_{2D}=4PL-8P^2(1-exp(-L/(2P)))   fit: ' ...
%	  sprintf([' P= %3.0f(\\pm %3.1f) nm'],...
%		  data.plength.nm.r2.m(param.condition(condition).bin),...
%		  data.plength.nm.r2.s(param.condition(condition).bin))]);
th=legend(hfit,sprintf(['P= %3.0f(\\pm %3.1f) nm'],...
		  data.plength.nm.r2.m(param.condition(condition).bin),...
		  data.plength.nm.r2.s(param.condition(condition).bin)),...
        'Location','NorthWest');
        %set(gca,'Ytick',[0:1000:24000]/param.NM_per_PX);
xlabel('length in nm')
ylabel(['<R^2>_{2D} (10^3 nm^2)']);
set(gcf,'visible','off');
%set(gcf,'visible','on');

%a=num2cell(round(get(gca,'YTick')*param.NM_per_PX/(10^3)));
%set(gca,'YTickLabel',a);
%sc=3;

%########################################################################
%<cos>
%########################################################################
function plotB(data,datahist,param,out,id,condition,lengths)
%make axes of plot current
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on

%plot(param.x.sampledat(param.condition(condition).bin).*[1,1],...
%     [0,1],'k--')


hdat = errorbar(param.x.sampledat,...
	 data.meancos.mean,...
	 data.meancos.std,'r.');
grid on
hfit =plot([0,param.x.sampleat],...
     avgcos(data.plength.nm.cos.m(param.condition(condition).bin),...
	    [0,param.x.sampleat]),'r');
axis tight
axx=axis;
axis(axx.*[0,1,1,0]+[0,0,0,1])
%setxtick(gca,10,param);  
%title(['<cos \theta>=exp(-l/(2P))   fit: '...
%       sprintf('P= %3.0f(\\pm%3.1f) nm',...
%	       data.plength.nm.cos.m(param.condition(condition).bin),...
%	       data.plength.nm.cos.s(param.condition(condition).bin))]);
legend(hfit,[ sprintf('P= %3.0f(\\pm%3.1f) nm',...
	       data.plength.nm.cos.m(param.condition(condition).bin),...
	       data.plength.nm.cos.s(param.condition(condition).bin))],'Location','NorthEast');
ylabel('<cos \theta>')
xlabel('length in nm')
set(gcf,'visible','off');
%########################################################################
%Histogram
%########################################################################
function plotC(data,datahist,param,out,id,condition,lengths)
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on
errorbar(param.x.sampledat,...
	 data.nelements.mean,...
	 data.nelements.std,'r.')
grid on
axis tight
axx=axis;
axis(axx.*[0,1,0,1])
%etxtick(gca,5,param);  
%title('Number of elements')
ylabel('#')
xlabel('length in nm')
set(gcf,'visible','off');
%########################################################################
%Histogram lengths
%########################################################################
function plotD(data,datahist,param,out,id,condition,lengths)
%%sp('function plotD not available at the moment')
%%turn;
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on

hist(lengths{condition},100);

%title(['# strands= ' num2str(length(lengths))]);
axx=axis;
axis(axx.*[1,0,1,1]+[0,200,1,1])
ylabel(sprintf('# of %d',length(lengths{condition})));
xlabel('lenght of strands (nm)');
set(gcf,'visible','off');
%########################################################################
%<angle^2>
%########################################################################
function plotE(data,datahist,param,out,id,condition,lengths)
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on

%plot(param.x.sampledat(param.condition(condition).bin).*[1,1],...
%     [0,param.x.sampleat(end)./data.plength.nm.p2.m(param.condition(condition).bin)],'k--')

hdat=errorbar(param.x.sampledat,...
	 data.meanthetasquared.mean,...
	 data.meanthetasquared.std,'r.');
grid on

hfit=plot([0,param.x.sampleat],...
     [0,param.x.sampleat]/data.plength.nm.p2.m(param.condition(condition).bin),...
     'r');

axis tight
axx=axis;
axis(axx.*[0,1,0,1])
%etxtick(gca,10,param);  
%title(['<\theta^2>=l/P   fit: ' ...
%       sprintf('P= %3.0f(\\pm%3.1f)',...
%	       data.plength.nm.p2.m(param.condition(condition).bin),...
%	       data.plength.nm.p2.s(param.condition(condition).bin)) 'nm']);
legend(hfit,[sprintf('P= %3.0f(\\pm%3.1f)',...
	       data.plength.nm.p2.m(param.condition(condition).bin),...
	       data.plength.nm.p2.s(param.condition(condition).bin)) 'nm'],...
          'Location','NorthWest');
ylabel('<\theta^2> (rad^2)')
xlabel('length in nm')
set(gcf,'visible','off');
%########################################################################
%<angle^4>/<angle^2>^2
%########################################################################
function plotF(data,datahist,param,out,id,condition,lengths)
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on
errorbar(param.x.sampledat,...
	 data.theta22divtheta4.mean,...
	 data.theta22divtheta4.std,'r.')
plot([0,max(param.x.sampledat)],1/3*[1,1],'k')
plot([0,max(param.x.sampledat)],mean(data.mx22div4)*[1,1],'r')
ylabel('')
grid on
axis tight;
axx=axis;
axis([axx(1),axx(2),.2,.6])
%etxtick(gca,10,param);  
ylabel('<\theta^2>^2/<\theta^4>')
xlabel('length in nm')
set(gcf,'visible','off');
%########################################################################
%D^4/L^4 = ( <R^4> - <R^2>^2 ) / L^4
%########################################################################
function plotG(data,datahist,param,out,id,condition,lengths)
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on

%plot(param.x.sampledat(param.condition(condition).bin).*[1,1],...
%     [0,max( data.D4.mean./(param.x.sampledat.^4) + data.D4.std./ (param.x.sampledat.^4) )],'k--')

errorbar(param.x.sampledat,...
	 data.D4.mean./(param.x.sampledat.^4),...
	 data.D4.std./(param.x.sampledat.^4),'r.')
p2=plot(param.x.sampleat,...
	meanr4minmeanr22ndim2d(data.plength.nm.d4n2d.m(param.condition(condition).bin),param.x.sampleat)./param.x.sampleat.^4,'r');
p3=plot(param.x.sampleat,meanr4minmeanr22ndim3d(data.plength.nm.d4n3d.m(param.condition(condition).bin) ,param.x.sampleat)./param.x.sampleat.^4,'r--');
hleg=legend([p2,p3],{
    ['P:Eq(8)= ' sprintf('%3.0f(\\pm%3.1f)',...
				     data.plength.nm.d4n2d.m(param.condition(condition).bin),...
				     data.plength.nm.d4n2d.s(param.condition(condition).bin) ) 'nm'],...
    ['P:Eq(7)= ' sprintf('%3.0f(\\pm%3.1f)',...
				     data.plength.nm.d4n3d.m(param.condition(condition).bin) ,...
				     data.plength.nm.d4n3d.s(param.condition(condition).bin) ) 'nm']},...
	    'Location','SouthEast' );
grid on
axis tight;
axx=axis;
axis([0,axx(2),0,axx(4)]);
set(hleg,'FontSize',5)
setxtick(gca,10,param);  
ylabel('<R^4>-<R^2>^2 / L^4');
xlabel('length in nm')
set(gcf,'visible','off');
%########################################################################
% Persistence lenght as function of L_max
%########################################################################
function plotH(data,datahist,param,out,id,condition,lengths)
axes(out.subplots.axes(condition,find(out.subplots.id==id)));

hold on
e1=errorbar(param.x.sampledat,...
	    data.plength.nm.r2.m,...
	    data.plength.nm.r2.s,'r.');
e2=errorbar(param.x.sampledat,...
	    data.plength.nm.cos.m,...
	    data.plength.nm.cos.s,'g.');
e3=errorbar(param.x.sampledat,...
	    data.plength.nm.p2.m,...
	    data.plength.nm.p2.s,'b.');
h=legend([e1,e2,e3],{'<r^2>','<cos>','<\theta^2>'},'orientation','horizontal','location','northeast','color','none');
grid on
%title(['Persistence length as function of the fit interval: [0,L_{max}]']);
ylabel('P (nm)');
xlabel('L_{max} (nm)')
axis tight
axx=axis;
axis(axx.*[0,1,0,1])
setxtick(gca,10,param);  
axx=axis;
axis(axx.*[1,1,0,0]+[0,0,10,60])
xlabel('L_{max} (nm)');
set(gcf,'visible','off');
%########################################################################
% 
%########################################################################
ln = length(param.global.dekker.samplelengthsnm);


function plotI(data,datahist,param,out,id,condition,lengths)
plotIJKL(data,datahist,param,out,id,condition,1)

function plotJ(data,datahist,param,out,id,condition,lengths)
plotIJKL(data,datahist,param,out,id,condition,2)

function plotK(data,datahist,param,out,id,condition,lengths)
plotIJKL(data,datahist,param,out,id,condition,3)

function plotL(data,datahist,param,out,id,condition,lengths)
plotIJKL(data,datahist,param,out,id,condition,4)

function plotIJKL(data,datahist,param,out,id,condition,ii)
if length(param.global.dekker.samplelengthsnm) < ii
return;
end
bins            = param.global.dekker.bins;
bincenters      = diff(bins)/2+bins(1:end-1);
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on
%fiterf
%plot(bincenters,-log(ttt(bincenters,diff(bins)/2,datahist.hist{ii}.fiterf,param.global.dekker.samplelengthsnm(ii),fac)),'r*')
%plot(bincenters,fga_erffit(bincenters,diff(bins)/2,datahist.hist{ii}.fiterf,param.global.dekker.samplelengthsnm(ii),2),'b*')


if strcmp(param.experiment,'artificaldata')
  %perstmp = str2num(param.condition(condition).name{1}(strfind(param.condition(condition).name{1},'_P')+[2,3]))*.34;
  tmp = regexp(param.condition(condition).name{1},'(\d+)','tokens');
  perstmp = str2num(tmp{2}{1})*.34;
  %fit erf with true
%  plot(bincenters,-log(ttt(bincenters,diff(bins)/2,perstmp,param.global.dekker.samplelengthsnm(ii),fac)),'c.')
%datahist.hist{ii}.fiterf



   plot(bincenters,...
       fga_erffit(bincenters,diff(bins)./2,perstmp,...
       param.global.dekker.samplelengthsnm(ii),2),'c-')
  
  
  hr=plot(bincenters,pla_erffit(bincenters,diff(bins)./2,datahist.hist{ii}.fiterf,param.global.dekker.samplelengthsnm(ii),2),'r-');
else
  hr=plot(bincenters,pla_erffit(bincenters,diff(bins)./2,datahist.hist{ii}.fiterf,param.global.dekker.samplelengthsnm(ii),2),'r-');

end


%parabola fit
%plot(bincenters,datahist.hist{ii}.par(1)*bincenters.^2+datahist.hist{ii}.par(2),'g.')
%data
%plot(datahist.hist{ii}.deflection*(pi/2/sqrt(2)),datahist.hist{ii}.mean,'.')
 %   datahist.hist{ii}.deflection
  %  datahist.hist{ii}.mean
   % datahist.hist{ii}.std
 errorbar(datahist.hist{ii}.deflection,datahist.hist{ii}.mean,datahist.hist{ii}.std,'r.')
try
 catch
plot(datahist.hist{ii}.deflection,datahist.hist{ii}.mean,'.r')
end



axis([0,pi/2,0,10]);%max(datahist.hist{ii}.mean)])
if strcmp(param.experiment,'EM474')
 % title(sprintf('L %.1f nm, P_{fit} %.1f nm',param.global.dekker.samplelengthsnm(ii),datahist.hist{ii}.fiterf))
 % h=legend('erf','par','data','Location','SouthEast');
else
  if 0
  title(sprintf('L %.1f nm, P_{fit} %.1f nm, P_{in}= %.1f nm',param.global.dekker.samplelengthsnm(ii),datahist.hist{ii}.fiterf,perstmp))
  h=legend('erf','true','par','data','Location','SouthEast');
  else
    h=legend(hr,sprintf('P= %.0f nm',datahist.hist{ii}.fiterf),'Location','SouthEast');
  end
end
%set(h,'color','none','box','off')
  
h=xlabel(['deflection (\pi rad)']);
%et(h,'interpreter','tex','fontsize',10);
h=ylabel(['-ln(G(\theta,' sprintf('%3.0f', ...
				  param.global.dekker.samplelengthsnm(ii)) ' nm))']);
set(h,'interpreter','tex','fontsize',10);
%set(gca,'xtick',[1:6]/6*pi)
%set(gca,'xticklabel',{'1/6','1/3','1/2','2/3','5/6','1'})
set(gca,'xtick',[1:3]/6*pi)
set(gca,'xticklabel',{'1/6','1/3','1/2'})
set(gcf,'visible','off');
%set(gcf,'visible','on')
%fdgsd=sdf




%########################################################################
%<angle^4>/<angle^2>^2
%########################################################################
%########################################################################
function plotM(data,datahist,param,out,id,condition,lengths)
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on
errorbar(param.x.sampledat,...
	 data.theta4divtheta22.mean,...
	 data.theta4divtheta22.std,'r.')
plot([0,max(param.x.sampledat)],3*[1,1],'k')
plot([0,max(param.x.sampledat)],1./mean(data.mx22div4)*[1,1],'r')
ylabel('')
grid on
axis tight;

axx=axis;
axis([axx(1),axx(2),2,3.5]);
set(gca,'Ytick',linspace(2,3.5,4));
set(gca,'YtickLabel',{'2.0','2.5','3.0','3.5'});
setxtick(gca,10);
ylabel('<\theta^4>/<\theta^2>^2')
xlabel('length in nm')
set(gcf,'visible','off');

%########################################################################
%<angle^4>/<angle^2>^2
%########################################################################
%########################################################################
function plotN(data,datahist,param,out,id,condition,lengths)
axes(out.subplots.axes(condition,find(out.subplots.id==id)));
hold on


x=zeros(size(datahist.hist,2),1);
y=zeros(size(datahist.hist,2),1);

for ii=1:size(datahist.hist,2)
    y(ii) = datahist.hist{ii}.fiterf;
    x(ii) = datahist.hist{ii}.L;
end
plot(x,y)

plot([min(x),max(x)],[1,1]*mean(data.plength.nm.cos.m),'r')
plot([min(x),max(x)],[1,1]*(mean(data.plength.nm.cos.m) + mean(data.plength.nm.cos.s)),'r--')
plot([min(x),max(x)],[1,1]*(mean(data.plength.nm.cos.m) - mean(data.plength.nm.cos.s)),'r--')


plot([min(x),max(x)],[1,1]*mean(data.plength.nm.r2.m),'g')
plot([min(x),max(x)],[1,1]*(mean(data.plength.nm.r2.m) + mean(data.plength.nm.r2.s)),'g--')
plot([min(x),max(x)],[1,1]*(mean(data.plength.nm.r2.m) - mean(data.plength.nm.r2.s)),'g--')

plot([min(x),max(x)],[1,1]*mean(data.plength.nm.p2.m),'b')
plot([min(x),max(x)],[1,1]*(mean(data.plength.nm.p2.m) + mean(data.plength.nm.p2.s)),'b--')
plot([min(x),max(x)],[1,1]*(mean(data.plength.nm.p2.m) - mean(data.plength.nm.p2.s)),'b--')

axis tight;
axx=axis;
axis([axx(1),axx(2),10,35]);

ylabel('P(L)(nm)')
xlabel('L(nm)')

%########################################################################
%########################################################################
%########################################################################
function out=ttt(theta,dtheta,P,L,fac)
out=(...
    erf(...
	sqrt(P./(fac*L)).*(theta+dtheta))-...
    erf(...
	sqrt(P./(fac*L)).*(theta-dtheta)));

%########################################################################

function [mx,st]=std_weighted(x,n)
mx=sum(x.*(n./repmat(sum(n),[size(x,1),1])));
st=sqrt(sum((x-repmat(mx,[size(x,1),1])).^2.*n./repmat(sum(n),[size(x,1),1])));

%########################################################################

function st=cov_weighted(x,y,n)
mx=sum(x.*(n./repmat(sum(n),[size(x,1),1])));
my=sum(y.*(n./repmat(sum(n),[size(y,1),1])));
st=sqrt(sum(...
    (((x-repmat(mx,[size(x,1),1])).*...
      (y-repmat(my,[size(x,1),1]))).*...
     (n./repmat(sum(n),[size(x,1),1])))));

%########################################################################

function sig=sigma22div4(x,y,sx,sy,sxy)
%meanthetasquared.mx,meanthetaquad.mx,meanthetasquared.st,meanthetaquad.st,cov22div4.st
sig=sqrt((4.*(x.^2))./(y.^2).*(sx .^2)+...
	 (x.^4) ./(y.^4).*(sy .^2)-...
	 (4.*(x.^3))./(y.^3).*(sxy.^2));

%########################################################################

function sig=sigmaR4minR22(x,y,sx,sy,sxy)
%meanrsquared.mx,meanrquad.mx, meanrsquared.st,meanrquad.st,covR4minR2
sig=sqrt(           (sy .^2)+...
		    4.*(x.^2).*(sx .^2)-...
		    4.*(x   ).*(sxy.^2));

%########################################################################

function setytick(axh)
global CUTOFF NSAMPLEWINDOWS NRANDOMDRAWS MAX_SAMPLELENGTH_PX NM_per_PX
a=num2cell(round(get(axh,'YTick')*NM_per_PX));
set(axh,'YTickLabel',a)
ylabel('length in nm')

%########################################################################

function setxtick(axh,steps,param)
%steps=10;
if (0)
nnm=round(param.MAX_SAMPLELENGTH_PX*param.NM_per_PX/steps);
set(axh,'XTick',[0:steps]*nnm/param.NM_per_PX);
a=num2cell([0:steps]*nnm);
set(axh,'XTickLabel',a)
xlabel('length in nm')
end

if 0
  lim=get(axh,'XLim');
  lim(1)=0;
  set(axh,'XLim',lim);
  r=474/1000;
  n=50;
  set(axh,'XTick',[ceil(lim(1)*r)/r:n/r:floor(lim(2)*r)/r]);
  a=num2cell([ceil(lim(1)*r):n:floor(lim(2)*r)]);
  set(axh,'XTickLabel',a)
  xlabel('length in nm')
end
%########################################################################

function f = meanr2f(q,x,y)
f =sum((meanr2(q,x)-y).^2);

%########################################################################

function f = meancosf(q,x,y)
f =(avgcos(q,x)-y);

%########################################################################

function f = avgcos(q,x)
f =exp(-x./(2*q));

%########################################################################

function f = meantheta2f(q,x,y)
f = ((x./q)-y);

%########################################################################

function f = meanr2(P,L)
f =(4.*P.*L-8.*P.^2.*(1-exp(-L./(2.*P))));

%########################################################################

function f = meanr2_3D(P,L)
f =(2*P.*L-2.*P.^2.*(1-exp(-L./P)));

%########################################################################


function f = meanr42D(P,L)
P2D=P*2;
f=( 8   .*L.^2.*P2D.^2 ...
    -30   .*L   .*P2D.^3 ...
    +87/2 .*      P2D.^4 ...
    -40/3 .*L   .*P2D.^3.*exp(-   L./P2D) ...
    -392/9 .*      P2D.^4.*exp(-   L./P2D) ...
    +1/18.*      P2D.^4.*exp(-4.*L./P2D));

%########################################################################

function f = meanr43D(P,L)
f=(20/3 .*L.^2.*P.^2 ...
   -208/9 .*L   .*P.^3 ...
   +856/27.*      P.^4 ...
   -8     .*L   .*P.^3.*exp(-   L./P) ...
   -32    .*      P.^4.*exp(-   L./P) ...
   +8/27 .*      P.^4.*exp(-3.*L./P));

%########################################################################
%########################################################################


function f= meanr4minmeanr22dim2d(P,L)
f=(meanr42D(P,L)-meanr2(P,L).^2);

%########################################################################

function f = lsqmeanr4minmeanr22dim2df(P,L,y)
f=(meanr4minmeanr22dim2d(P,L)-mean(y,1));

%########################################################################

function f= meanr4minmeanr22dim3d(P,L)
f=(meanr43D(P,L)-meanr2_3D(P,L).^2);

%########################################################################

function f = lsqmeanr4minmeanr22dim3df(P,L,y)
f=(meanr4minmeanr22dim3d(P,L)-mean(y,1));

%########################################################################
%########################################################################
%normalised versions

%########################################################################

function f= meanr4minmeanr22ndim2d(P,L)
f=(meanr42D(P,L)-meanr2(P,L).^2);

%########################################################################

function f = lsqmeanr4minmeanr22ndim2df(P,L,y)
f=sqrt(((meanr4minmeanr22dim2d(P,L)-mean(y,1)).^2)./L.^4);

%########################################################################

function f= meanr4minmeanr22ndim3d(P,L)
f=(meanr43D(P,L)-meanr2_3D(P,L).^2);

%########################################################################

function f = lsqmeanr4minmeanr22ndim3df(P,L,y)
f=sqrt(((meanr4minmeanr22dim3d(P,L)-mean(y,1)).^2)./L.^4);

%########################################################################
%########################################################################
%########################################################################

function [indices,d,dind]=sample(minll,maxll,nn,kk)
%minll minium interval length
%maxll maxium interval length
%nn number of intervallengths
%kk number of realisations
indices=round(logspace(log10(minll),log10(maxll),nn));

for jj=1:kk
  done=0;
  ii=1;
  m=ones(1,nn);
  d{jj}=[];
  dind{jj}=[];
  while ~done
    nt=sum(m);
    ind=round(rand(1)*nt-.5)+1;
    if ~ind
      ind=1;
    end
    d{jj}=[d{jj},indices(ind)];
    dind{jj}=[dind{jj},ind];
    if minll >= (maxll-sum(d{jj}))
      done=1;
    elseif all(m==0)
      done =1;
    else
      m = (indices < (maxll-sum(d{jj})));
    end
  end
end


%########################################################################
%########################################################################
%########################################################################

function out = fitpowerlaw(L,R)
global x;
global y;
x=L;
y=R;
x0=[1,.6,44,1];
options = optimset('LargeScale','off','TolX',1e-9,'MaxFunEvals',10000,'MaxIter',10000);
out= fminunc(@lsq,x0,options);


%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
%########################################################################
%########################################################################
%########################################################################

function out = lsq(param)
global x;
global y;
out=sum(((y)-(meanrpowerlaw(x,param))).^2);

%########################################################################
%########################################################################
%########################################################################


function out=meanrpowerlaw(x,param)
out=param(4)*...
    (((x./param(3)).^param(1)).*...
     ((1+(x./param(3))).^(param(2)-param(1))));

%########################################################################
%########################################################################
%########################################################################

function ng=estimatep(p_theta,p_r)
global NM_per_PX BINWIDTH_IN_PX

fg=mean([p_theta,p_r])./(BINWIDTH_IN_PX);
in=round(mean([p_theta,p_r])./(BINWIDTH_IN_PX)+([-1:1]));
in=in(abs(in-fg)<1.501);%take indices within a distance of 1.51 bin
			%from original estimate
			ng=mean([p_theta(in),p_r(in)])*NM_per_PX;
