function [tracks,lengths] = pla_sample_tracks(data,param,condition);

%path
for jj = 1 : size(data.path,2)
    fprintf('\r %3d / %3d',jj,size(data.path,2));
    
    %conversion factor form px to nm
    nmperpx = data.resolution(jj).NM_per_PX;
    
    cutoff = param.global.cutoff;
    
    %resample at nm step length
    tracks{jj} = sampleatnminterval(data.path{jj}{end},nmperpx,cutoff);
    lengths(jj) = tracks{jj}.l;
    
end

function out = sampleatnminterval(crdspx,nmperpx,cutoff);


%x and y in nm
x = crdspx(:,1)*nmperpx;
y = crdspx(:,2)*nmperpx;

%number of segments (+1) in pixels
n = length( x );

if( n < cutoff*5)
    %to small strand ignore in further analysis
    out=[];
else
    
    SAMPFAC =  1./median(diff(x).^2+diff(y).^2);%input samples per pixel
    OVERFAC = 1000; %oversamplingsfactor, i.e. approx samples per pixel
    SCALEFAC = ceil(OVERFAC/SAMPFAC);% next line...
    %fprintf('%f sdghfksjdfh gkjsdf\n\n\n', SCALEFAC);
    %correction factor between input
    %samples per pixel and desired
    %number of samples each pixel
    
    sampling = linspace( 1 , n , n*SCALEFAC);
    a.x      = spline( 1:n , x , sampling );
    a.y      = spline( 1:n , y , sampling );
    a.l      = cumsum( [0,sqrt( diff( a.x ).^2 + diff( a.y ).^2)]);
    
    %get spline as function of the length along the contour
    p.x =  spline( a.l , a.x );
    p.y =  spline( a.l , a.y );

    %    p.dx = fnder( p.x );
    p.dx = pla_fnder(p.x);

    %    p.dy = fnder( p.y );
    p.dy = pla_fnder( p.y );
    
    %sample interval each nm
    sampling = [round(cutoff+1) : 1 : ( floor(a.l(end)) - round(cutoff) )];
    out.x    = ppval( p.x    , sampling );
    out.y    = ppval( p.y    , sampling );
    out.dx   = ppval( p.dx   , sampling );
    out.dy   = ppval( p.dy   , sampling );
    out.phi  = atan2( out.dy , out.dx   );
    out.l    = a.l(end);
    
end


